import depoc
import json
import click

from rich.console import Console


console = Console()
        

@click.command(help='Logout of your account')
def logout() -> None:
    with open(depoc.token_path, 'w') as f:
        console.clear()
        console.print('🫂  bye bye, see you soon.')
        json.dump({'token': None}, f)



import depoc
import click
import time
import sys

from typing import Any

from .utils._response import _handle_response
from .utils._format import spinner, page_summary, _format_contact

from rich.console import Console
from rich.panel import Panel
from rich.prompt import Prompt


client = depoc.DepocClient()
console = Console()


@click.group(invoke_without_command=True)
@click.pass_context
@click.option('-l', '--limit', default=50)
@click.option('-p', '--page', default=0)
def supplier(cxt, limit: int, page: int) -> None:
    ''' Manage suppliers. '''
    if cxt.invoked_subcommand is None:
        service = client.suppliers.all
        
        if response := _handle_response(service, limit=limit, page=page):
            page_summary(response)

            for obj in response.results:
                _format_contact(obj, f'{obj.legal_name}')


@supplier.command
def create() -> None:
    ''' Create supplier. '''
    panel = Panel('[bold]+ ADD NEW SUPPLIER')
    console.print(panel)

    data: dict[str, Any] = {}
    data.update({'code': Prompt.ask('🆔 Code', default=None)})
    console.rule('',style=None, align='left')
    data.update({'legal_name': Prompt.ask('✏️  Legal Name', default=None)})
    console.rule('',style=None, align='left')
    data.update({'trade_name': Prompt.ask('✏️  Trade Name', default=None)})
    console.rule('',style=None, align='left')
    data.update({'cnpj': Prompt.ask('📋 CNPJ', default=None)})
    console.rule('',style=None, align='left')
    data.update({'ie': Prompt.ask('📋 IE', default=None)})
    console.rule('',style=None, align='left')
    data.update({'im': Prompt.ask('📋 IM', default=None)})
    console.rule('',style=None, align='left')
    data.update({'phone': Prompt.ask('📱 Phone', default=None)})
    console.rule('',style=None, align='left')
    data.update({'email': Prompt.ask('✉️  Email', default=None)})
    console.rule('',style=None, align='left')
    data.update({'postcode': Prompt.ask('📮 Postal Code', default=None)})
    console.rule('',style=None, align='left')
    data.update({'city': Prompt.ask('🏙️  City', default=None)})
    console.rule('',style=None, align='left')
    data.update({'state': Prompt.ask('🗺️  State', default=None)})
    console.rule('',style=None, align='left')
    data.update({'address': Prompt.ask('📍 Address', default=None)})
    console.rule('',style=None, align='left')
    data.update({'notes': Prompt.ask('🗒️  Notes', default=None)})
    console.rule('',style=None, align='left')

    service = client.suppliers.create

    if obj := _handle_response(service, data):
        _format_contact(obj, obj.legal_name)

@supplier.command
@click.argument('id')
def get(id: str) -> None:
    ''' Retrieve an specific supplier. '''
    service = client.suppliers.get

    if obj := _handle_response(service, resource_id=id):
        _format_contact(obj, obj.legal_name)

@supplier.command
@click.argument('id')
@click.option('-c', '--code')
@click.option('-l', '--legal_name')
@click.option('-t', '--trade_name')
@click.option('-p', '--phone')
@click.option('-e', '--email')
@click.option('-s', '--state')
@click.option('--cnpj')
@click.option('--ie')
@click.option('--im')
@click.option('--notes')
@click.option('--city')
@click.option('--address')
@click.option('--postcode')
@click.option('-A', '--activate', is_flag=True)
def update(
    id: str,
    code: str,
    legal_name: str,
    trade_name: str,
    cnpj: str,
    ie: str,
    im: str,
    notes: str,
    phone: str,
    email: str,
    postcode: str,
    city: str,
    state: str,
    address: str,
    activate: bool,
    ) -> None:
    ''' Update an specific customer. '''

    # unable to update a field to an empty value
    data: dict[str, Any] = {}
    data.update({'code': code}) if code else ''
    data.update({'legal_name': legal_name}) if legal_name else ''
    data.update({'trade_name': trade_name}) if trade_name else ''
    data.update({'cnpj': cnpj}) if cnpj else ''
    data.update({'ie': ie}) if ie else ''
    data.update({'im': im}) if im else ''
    data.update({'notes': notes}) if notes else ''
    data.update({'phone': phone}) if phone else ''
    data.update({'email': email}) if email else ''
    data.update({'postcode': postcode}) if postcode else ''
    data.update({'city': city}) if city else ''
    data.update({'state': state}) if state else ''
    data.update({'address': address}) if address else ''
    data.update({'is_active': True}) if activate else ''

    service = client.suppliers.update

    if obj := _handle_response(service, data, resource_id=id):
        _format_contact(obj, 'UPDATED', update=True)

@supplier.command
@click.argument('ids', nargs=-1)
def delete(ids: str) -> None:
    ''' Delete an specific supplier. '''
    service = client.suppliers.delete

    while True:
        prompt = click.style('Proceed to deletion? [y/n] ', fg='red')
        confirmation = input(prompt)
        if confirmation == 'n':
            sys.exit(0)
        elif confirmation == 'y':
            break

    if len(ids) > 1:
        spinner()

    for id in ids:
        time.sleep(0.5)
        if _handle_response(service, resource_id=id):
            console.print('✅ Supplier inactivated')



import click
import sys
import itertools
import time
import math

from datetime import datetime

from rich.console import Console, Group
from rich.panel import Panel
from rich.table import Table

from typing import Literal

from depoc.objects.base import DepocObject

console = Console()

emojis = {
    'ID': ':white_medium_star:',
    'Name': '✏️ ',
    'Email': '✉️ ',
    'Phone': '📱',
    'Username': '👤',
    'Active': '🟢',
    'Staff': '💼',
    'Last Login': '🕒',
    'Date Joined': '📅',
    'Code':'🆔',
    'Gender': '⚧️ ',
    'CPF': '📋',
    'CNPJ': '📋',
    'IE': '📋',
    'IM': '📋',
    'Postcode': '📮',
    'City': '🏙️ ',
    'State': '🗺️ ',
    'Address': '📍',
    'Amount Spent': '💸',
    'Number Of Orders': '🛒',
    'Created At': '📅',
    'Updated At': '🕒',
    'Category': '📂',
    'Issued At': '📅',
    'Due At': '📅',
    'Paid At': '📅',
    'Updated At': '🕒',
    'Total Amount': '💰',
    'Amount Paid': '💸',
    'Outstanding Balance': '💵',
    'Payment Type': '🏧',
    'Payment Method': '💳',
    'Status': '🟢',
    'Recurrence': '⏳',
    'Installment Count': '⏰',
    'Due Weekday': '📆',
    'Due Day Of Month': '📆',
    'Reference': '📎',
}


def _format_category(obj, highlight: bool = False):
    table = Table(
        show_header=True,
        show_footer=True,
        box=None,
        expand=True,
        )
    
    table.add_column('', justify='left', no_wrap=True)
    table.add_column('', justify='right', no_wrap=True)

    table.add_row(obj.name, obj.id)

    border_style = 'none'
    style = 'none'

    if highlight:
        border_style='green'
        style='green'

    profile = Panel(
        table,
        title_align='left',
        subtitle_align='left',
        border_style=border_style,
        style=style,
    )
    console.print(profile)


def _format_transactions(
        obj: DepocObject,
        title: str,
        update: bool = False,
        detail: bool = False
    ):

    timestamp = datetime.fromisoformat(obj.timestamp)
    obj.timestamp = timestamp.strftime('%Y-%m-%d %H:%M:%S')

    table = Table(
        show_header=True,
        show_footer=True,
        box=None,
        expand=True,
        title=f'R${float(obj.amount):,.2f}',
        caption=obj.timestamp,
        title_justify='right',
    )

    table.add_column('', justify='left', no_wrap=True)
    table.add_column('', justify='right', no_wrap=True)

    data = obj.to_dict()
    data.pop('id', None)
    data.pop('description', None)
    data.pop('timestamp', None)
    data.pop('type', None)
    data.pop('account', None)
    data.pop('amount', None)

    if not detail:
        data.pop('operator', None)
        data.pop('linked', None)

    for k, v in data.items():
        k = k.replace('_', ' ').title()
        k = k.replace('Is ', '')
        k = k.upper() if k in ('Cpf', 'Cnpj', 'Ie', 'Im') else k

        if isinstance(v, DepocObject):
            if hasattr(v, 'name'):
                v = v.name
        
        if v:
            table.add_row(f'{k}: ', f'{v}')
    
    description = Table(
        show_header=False,
        show_footer=True,
        box=None,
        expand=True,
        title=obj.description,
        title_justify='center',
    )
    description.add_column('', justify='left', no_wrap=True)
    
    group = Group(table, description)

    style = 'none'
    panel_title = f'{title}'
    subtitle = f'[bold]{obj.id}'
    border_style = 'none'

    if update:
        style = 'green'
        panel_title = f'[bold][green]{title}'
        subtitle = f'[green]{obj.id}'
        border_style = 'none'
    elif obj.type == 'credit':
        border_style = 'green'
    elif obj.type == 'debit':
        border_style = 'red'

    profile = Panel(
        group,
        title=panel_title,
        title_align='left', 
        subtitle=subtitle,
        subtitle_align='left',
        style=style,
        border_style=border_style,
    )

    console.print(profile)



def _format_payments(
        obj: DepocObject,
        title: str,
        update: bool = False,
        detail: bool = False
    ):

    table_title = f'[bold]R${float(obj.outstanding_balance):,.2f}'

    table = Table(
        show_header=True,
        show_footer=True,
        box=None,
        expand=True,
        title=table_title,
        caption=f'{obj.due_at}',
        title_justify='right',
        caption_justify='center'
    )

    table.add_column('', justify='left', no_wrap=True)
    table.add_column('', justify='left', no_wrap=True)

    data = obj.to_dict()
    data.pop('id', None)
    data.pop('notes', None)
    data.pop('contact', None)
    data.pop('payment_type', None)

    if not detail:
        data.pop('status', None)
        data.pop('due_at', None)
        data.pop('updated_at', None)
        data.pop('payment_method', None)
        data.pop('recurrence', None)
        data.pop('reference', None)

    for k, v in data.items():
        if k in 'updated_at':
            v = v[:10] if v is not None else 'null'

        k = k.replace('_', ' ').title()
        k = k.replace('Is ', '')
        k = k.upper() if k in ('Cpf', 'Cnpj', 'Ie', 'Im') else k
        
        if v:
            table.add_row(f'{emojis[k]} {k}: ', f'{v}')
    
    notes = Table(
        show_header=False,
        show_footer=True,
        box=None,
        expand=True,
        title=obj.notes,
        title_justify='center',
    )
    notes.add_column('', justify='left', no_wrap=True)
    
    group = Group(table, notes)

    style = 'none'
    panel_title = f'[bold]{title}'
    subtitle = f'[bold]{obj.id}'
    border_style = 'none'

    if update:
        style = 'green'
        panel_title = f'[bold][green]{title}'
        subtitle = f'[green]{obj.id}'
        border_style = 'none'
    elif obj.status == 'overdue':
        style = 'none'
        panel_title = f'[bold]{title}'
        subtitle = f'[bold]{obj.id}'
        border_style = 'red'
    elif obj.status == 'paid':
        style = 'none'
        panel_title = f'[bold]{title}'
        subtitle = f'[bold]{obj.id}'
        border_style = 'green'

    profile = Panel(
        group,
        title=panel_title,
        title_align='left', 
        subtitle=subtitle,
        subtitle_align='left',
        style=style,
        border_style=border_style,
    )

    console.print(profile)


def _format_contact(
        obj: DepocObject,
        title: str,
        update: bool = False,
    ):

    if hasattr(obj, 'alias'):
        table_title = obj.alias
        caption = 'customer'
    elif hasattr(obj, 'trade_name'):
        table_title = obj.trade_name
        caption = 'supplier'

    table = Table(
        show_header=True,
        show_footer=True,
        box=None,
        expand=True,
        title=table_title,
        title_justify='right',
        caption=caption,
        caption_justify='right'
    )

    table.add_column('', justify='left', no_wrap=True)
    table.add_column('', justify='left', no_wrap=True)

    data = obj.to_dict()
    data.pop('id', None)
    data.pop('name', None)
    data.pop('alias', None)
    data.pop('legal_name', None)
    data.pop('trade_name', None)
    data.pop('notes', None)

    for k, v in data.items():
        if k in ('last_login', 'date_joined', 'created_at'):
            v = v[:10] if v is not None else 'null'

        k = k.replace('_', ' ').title()
        k = k.replace('Is ', '')
        k = k.upper() if k in ('Cpf', 'Cnpj', 'Ie', 'Im') else k
        
        if v and obj.is_active:
            table.add_row(f'{emojis[k]} {k}: ', f'{v}')
    
    group = Group(table)

    if obj.notes:
        notes = Table(
            show_header=False,
            show_footer=True,
            box=None,
            expand=True,
            title=obj.notes,
            title_justify='center',
        )
        notes.add_column('', justify='left', no_wrap=True)
        group = Group(table, notes)

    if update:
        style = 'green'
        panel_title = f'[bold][green]{title}'
        subtitle = f'[green]{obj.id}'
    elif not obj.is_active:
        style = 'bright_red'
        panel_title = f'[bold][bright_red]{title}'
        subtitle = f'[bright_red]{obj.id}'
    else:
        style = 'none'
        panel_title = f'[bold]{title}'
        subtitle = f'[blue]{obj.id}'

    profile = Panel(
        group,
        title=panel_title,
        title_align='left', 
        subtitle=subtitle,
        subtitle_align='left',
        style=style
    )

    console.print(profile)


def _format_profile(
        obj: DepocObject,
        title: str,
        columns: int = 2,
        update: bool = False,
        delete: bool = False,
    ):
    table = Table(show_header=True, show_footer=True, box=None, expand=True)

    for _ in range(columns):
        table.add_column('', justify='left', no_wrap=True)

    data = obj.to_dict()

    for k, v in data.items():
        if k in ('last_login', 'date_joined'):
            v = v[:10] if v is not None else 'null'

        k = k.replace('_', ' ').title()
        k = k.replace('Is ', '')
        k = k.upper() if k == 'Id' else k
        
        table.add_row(f'{emojis[k]} {k}: ', f'{v}')

    if update:
        style = 'green'
    elif delete:
        style = 'red'
    else:
        style = 'none'

    profile = Panel(table, title=f'[bold]{title}', title_align='left', style=style)

    console = Console()
    console.print(profile)


def _format_response(
        obj: DepocObject,
        title: str,
        header: str,
        highlight: str | None = None,
        color: Literal[
            'red',
            'green',
            'yellow',
            'blue',
            'magenta',
            'cyan',
        ] = 'yellow',
        remove: list[str] | None = None,
    ):
    
    try:
        if obj.is_active == False:
            color = 'red'
    except AttributeError:
        pass

    title = click.style(f'{title.upper():-<50}', fg=color, bold=True)
    header = click.style(f'\n{header:>50}', bold=True)

    if highlight:
        if len(highlight) > 50:
            highlight = highlight[:50] if len(highlight) > 50 else None
        highlight = click.style(f'\n{highlight:>50}', bold=True)

    data = obj.to_dict()
    body: str = ''

    if remove:
        for item in remove:
            data.pop(item)

    for k, v in data.items():
        k = k.replace('_', ' ').title()
        k = k.upper() if k == 'Id' else k

        if isinstance(v, DepocObject):
            if hasattr(v, 'name'):
                v = v.name

        body += f'\n{k}: {v}'

    response = (
        f'{title}'
        f'{header}'
        f'{highlight if highlight else ''}'
        f'{body}'
    )
    click.echo(response)


def spinner() -> None:
    spinner_cycle = itertools.cycle(['-', '\\', '|', '/'])
    for _ in range(20):
        sys.stdout.write(f'\rDeleting {next(spinner_cycle)} ')
        sys.stdout.flush()
        time.sleep(0.1)
    click.echo('')


def page_summary(response: DepocObject):
    total_pages = math.ceil(response.count / 50)
    results_count = len(response.results)
    current_page_number = 1

    if response.next:
        next_page_number = response.next[-1]
        current_page_number = int(next_page_number) - 1
    elif response.previous and not response.next:
        current_page_number = total_pages

    message = (
        f'\n[Page {current_page_number}/{total_pages}] '
        f'Showing {results_count} results (Total: {response.count})\n'
    )

    click.echo(message)



import depoc
import click

from rich.console import Console
from rich.panel import Panel
from rich.table import Table

from .utils._response import _handle_response
from .utils._format import _format_profile


client = depoc.DepocClient()


@click.command
def me() -> None:
    ''' Get the current user's data'''
    service = client.me.get

    if obj := _handle_response(service):
        _format_profile(obj, 'USER PROFILE')



from .account import account
from .contact import contact
from .customer import customer
from .login import login
from .logout import logout
from .me import me
from .owner import owner
from .payable import payable
from .receivable import receivable
from .report import report
from .supplier import supplier
from .balance import balance



import click

from typing import Any

from depoc.objects.base import DepocObject
from depoc.utils._error import APIError


def _handle_response(
        service: Any,
        data: dict[str, Any] | None = None,
        resource_id: str | None = None,
        **params,
    ) -> DepocObject | None:
    '''
    Handles the response for a service method call.

    The `service` argument should be a method of a
    resourcein the DepocClient instance.

    Examples of valid service methods:
    - `client.customer.create` 
    - `client.receivable.all`
    '''
    try:
        if data and resource_id:
            response = service(data, resource_id)
        elif data:
            response = service(data)
        elif resource_id:
            response = service(resource_id)
        else:
            response = service(**params)

        return response
    
    except APIError as e:
        click.echo(str(e))
    return None



import json
import os

from depoc.core.requestor import Requestor
from depoc.core.client import DepocClient
from depoc.core.auth import Connection

from depoc.services.user import User
from depoc.services.owner import Owner
from depoc.services.account import Account
from depoc.services.business import Business
from depoc.services.customer import Customer
from depoc.services.supplier import Supplier

from appdirs import user_data_dir # type: ignore


# Constants
BASE_URL: str = 'https://api.depoc.com.br'
APP_NAME = 'depoc'


config_dir = user_data_dir(APP_NAME)
os.makedirs(config_dir, exist_ok=True)
token_path = os.path.join(config_dir, 'token.json')

token: str | None = None

try:
    with open(token_path, 'r') as f:
        data: dict = json.load(f)
        token = data.get('token')
except (FileNotFoundError, json.JSONDecodeError, KeyError) as e:
    pass



import depoc
import click

from typing import Any

from .utils._response import _handle_response
from .utils._format import _format_profile


client = depoc.DepocClient()


@click.group(invoke_without_command=True)
@click.pass_context
def owner(ctx) -> None:
    ''' Owner - retrieve and update. '''
    if ctx.invoked_subcommand is None:
        service = client.owner.get

        if obj := _handle_response(service):
            _format_profile(obj, 'OWNER PROFILE')

@owner.command
@click.option('-n', '--name')
@click.option('-e', '--email')
@click.option('-p', '--phone')
def update(name: str, email: str, phone: str) -> None:
    ''' Update owner. '''
    data: dict[str, Any] = {}
    data.update({'name': name}) if name else None
    data.update({'email': email}) if email else None
    data.update({'phone': phone}) if phone else None
    
    service = client.owner.update

    if obj := _handle_response(service, data):
        _format_profile(obj, '[green]UPDATED', update=True)



import depoc
import click
import sys
import time

from typing import Any

from rich.console import Console
from rich.panel import Panel
from rich.prompt import Prompt

from .utils._response import _handle_response
from .utils._format import (
    spinner,
    page_summary,
    _format_payments,
    _format_transactions,
)


client = depoc.DepocClient()
console = Console()


@click.group(invoke_without_command=True)
@click.option('-l', '--limit', default=50)
@click.option('-p', '--page', default=0)
@click.option('--all', is_flag=True)
@click.option('--detail', is_flag=True)
@click.pass_context
def receivable(
    ctx,
    limit: int,
    page: int,
    all: bool = False,
    detail: bool = False
) -> None:
    ''' Manage receivables. '''
    if ctx.invoked_subcommand is None:
        service = client.receivables.all

        if not all and limit != 50: 
            console.print((
                '🚨 [bold][-l][/bold] & [bold][--limit][/bold] requires '
                'the [yellow]--all[/yellow] flag.'
            ))
            sys.exit()

        if not all and page > 0: 
            console.print((
                '🚨 [bold][-p][/bold] & [bold][--page][/bold] requires '
                'the [yellow]--all[/yellow] flag.'
            ))
            sys.exit()

        if response := _handle_response(service, limit=limit, page=page):
            results = sorted(response.results, key=lambda obj: obj.due_at)

            if not all:
                results = [obj for obj in results if obj.status != 'paid']
                results_count = len(results)

                # This page summary data needs to be
                # reviewed to accommodate different cases
                click.echo((
                    f'\n[Page {1}/{1}] '
                    f'Showing {results_count} results '
                    f'(Total: {results_count})\n'
                ))
                console.rule(
                    '[bold]OUTSTANDING RECEIVABLES',
                    align='center',
                    style='bold',
                )
            else:
                page_summary(response)
                console.rule('ALL RECEIVABLES', align='center', style='none')
            
            console.print('')

            total_receivable: float = 0
            for obj in results:
                total_receivable += float(obj.outstanding_balance)             
                _format_payments(obj, obj.contact, detail=detail)

            format_total = f'[yellow]R${total_receivable:,.2f}'
            message = f'\n{'[bold]💵 Total to be received: ' + format_total}\n'
            console.print(message)

@receivable.command
def create() -> None:
    ''' Create receivable. '''
    panel = Panel('[bold]+ ADD NEW RECEIVABLE')
    console.print(panel)

    data: dict[str, Any] = {}
    data.update({'total_amount': Prompt.ask('💰 Total Amount R$', default=None)})
    console.rule('',style=None, align='left')
    data.update({'due_at': Prompt.ask('📅 Due At', default=None)})
    console.rule('',style=None, align='left')
    data.update({'issued_at': Prompt.ask('📅 Issued At', default=None)})
    console.rule('',style=None, align='left')
    data.update({'payment_method': Prompt.ask('💳 Payment Method', default=None)})
    console.rule('',style=None, align='left')
    recurrence = Prompt.ask('⏳ Recurrence', default=None, choices=['once', 'weekly', 'monthly', 'installments'])
    data.update({'recurrence': recurrence})
    console.rule('',style=None, align='left')

    if recurrence == 'monthly':
        console.rule('',style=None, align='left')
        data.update({'due_day_of_month': Prompt.ask('📆 Due Day of Month', default=None)})
    elif recurrence == 'weekly':
        console.rule('',style=None, align='left')
        data.update({'due_weekday': Prompt.ask('📆 Due Weekday', default=None)})
    elif recurrence == 'installments':
        console.rule('',style=None, align='left')
        data.update({'installment_count': Prompt.ask('⏰ Installments', default=None)})
        console.rule('',style=None, align='left')
        data.update({'due_day_of_month': Prompt.ask('📆 Due Day of Month', default=None)})


    data.update({'category': Prompt.ask('📂 Category ID', default=None)})
    console.rule('',style=None, align='left')
    data.update({'contact': Prompt.ask('🆔 Contact ID', default=None)})
    console.rule('',style=None, align='left')
    data.update({'reference': Prompt.ask('📎 Reference Number', default=None)})
    console.rule('',style=None, align='left')
    data.update({'notes': Prompt.ask('🗒️  Notes', default=None)})
    console.rule('',style=None, align='left')

    service = client.receivables.create

    if obj := _handle_response(service, data):
        _format_payments(obj, obj.contact, detail=True)

@receivable.command
@click.argument('id')
def get(id: str) -> None:
    ''' Retrieve an specific receivable. '''
    service = client.receivables.get
    if obj := _handle_response(service, resource_id=id):
        _format_payments(obj, obj.contact, detail=True)

@receivable.command
@click.argument('id')
@click.option('-c', '--contact')
@click.option('-i', '--issued-at')
@click.option('-d', '--due-at')
@click.option('-t', '--total-amount')
@click.option('-p', '--payment-method')
@click.option('-r', '--reference')
@click.option('-n', '--notes')
@click.option('--category')
def update(
    id: str,
    contact: str,
    category: str,
    issued_at: str,
    due_at: str,
    total_amount: float,
    payment_method: str,
    reference: str,
    notes: str,
    ) -> None:
    ''' Update an specific payable. '''
    # Bug to fix: unable to update a field to an empty value
    data: dict[str, Any] = {}
    data.update({'contact': contact}) if contact else None
    data.update({'category': category}) if category else None
    data.update({'issued_at': issued_at}) if issued_at else None
    data.update({'due_at': due_at}) if due_at else None
    data.update({'total_amount': total_amount}) if total_amount else None
    data.update({'payment_method': payment_method}) if payment_method else None
    data.update({'reference': reference}) if reference else None
    data.update({'notes': notes}) if notes else None

    service = client.receivables.update
    if obj := _handle_response(service, data, resource_id=id):
        _format_payments(obj, 'UPDATED', update=True, detail=True)

@receivable.command
@click.argument('ids', nargs=-1)
def delete(ids: str) -> None:
    ''' Delete an specific receivable. '''
    service = client.receivables.delete

    while True:
        prompt = click.style('Proceed to deletion? [y/n] ', fg='red')
        confirmation = input(prompt)
        if confirmation == 'n':
            sys.exit(0)
        elif confirmation == 'y':
            break

    if len(ids) > 1:
        spinner()

    for id in ids:
        time.sleep(0.5)
        if obj := _handle_response(service, resource_id=id):
            console.print('✅ Receivable deleted')

@receivable.command
@click.argument('id')
@click.option('--amount', required=True)
@click.option('--account', required=True)
def settle(id: str, amount: float, account: str) -> None:
    ''' Settle a receivable '''
    data: dict[str, Any] = {'amount': amount, 'account': account}

    service = client.receivable_settle.create

    if obj := _handle_response(service, data, id):
        title = f'\n{obj.account.name} {obj.type}'.upper()
        _format_transactions(obj, title)

@receivable.command
@click.argument('search', required=False)
@click.option('-d', '--date')
@click.option('-s', '--start-date')
@click.option('-e', '--end-date')
@click.option('-l', '--limit', default=50)
@click.pass_context
def search(
    ctx,
    search: str,
    date: str,
    start_date: str,
    end_date: str,
    limit: int,
    ) -> None:
    ''' Filter receivables. '''
    if not any([search, date, start_date, end_date]):
        click.echo(ctx.get_help())
        sys.exit(0)

    service = client.receivables.filter

    if response := _handle_response(
        service,
        search=search,
        date=date,
        start_date=start_date,
        end_date=end_date,
        limit=limit,
    ):
        results = sorted(response.results, key=lambda obj: obj.due_at)

        page_summary(response)

        total_receivable: float = 0
        for obj in results:
            total_receivable += float(obj.outstanding_balance)
            _format_payments(obj, obj.contact)

        click.echo(click.style(f'\n{'':-<49}', bold=True))
        format_total = f'[yellow]R${total_receivable:,.2f}'
        message = f'\n{'[bold]💵 Total to be received: ' + format_total}\n'
        console.print(message)



from depoc.resources.methods import Finder, Create, Retrieve, Update, Delete
from depoc.objects.payment import PaymentObject, PaymentSettleObject


class Payable(
    Finder[PaymentObject],
    Create[PaymentObject],
    Retrieve[PaymentObject],
    Update[PaymentObject],
    Delete[PaymentObject],
):
    obj = PaymentObject
    endpoint = 'payables'
    label = 'payment'


class PayableSettle(
    Create[PaymentSettleObject]
):
    obj = PaymentSettleObject
    endpoint = 'payables/<id>/settle'
    label = 'transaction'



from depoc.resources.methods import Create, Retrieve, Update, Delete
from depoc.objects.supplier import SupplierObject


class Supplier(
    Create[SupplierObject],
    Retrieve[SupplierObject],
    Update[SupplierObject],
    Delete[SupplierObject],
):
    obj = SupplierObject
    endpoint = 'contacts/suppliers'
    label = 'supplier'



from .base import DepocObject


class FinancialAccountObject(DepocObject):
    '''Represents a financial account with balance and status.'''

    id: str
    '''Unique identifier for the account.'''

    name: str
    '''Name of the financial account.'''

    balance: str
    '''Current balance of the account.'''

    created_at: str
    '''Timestamp of when the account was created.'''

    is_active: bool
    '''Indicates if the account is active.'''


class FinancialCategoryObject(DepocObject):
    '''Represents a category for financial transactions.'''

    id: str
    '''Unique identifier for the category.'''

    name: str
    '''Name of the financial category.'''

    parent: str
    '''ID of the parent category, if any.'''

    is_active: str
    '''Indicates if the category is active.'''


class FinancialTransactionObject(DepocObject):
    '''Represents a financial transaction record.'''

    id: str
    '''Unique identifier for the transaction.'''

    category: dict
    '''ID of the associated financial category.'''

    operator: dict
    '''ID of the user or entity that performed the transaction.'''

    account: dict
    '''ID of the account involved in the transaction.'''

    contact: dict
    '''ID of the related contact or client.'''

    payment: str
    '''ID of the related Payment (payable or receivable).'''

    linked: str
    '''ID of any linked transaction.'''

    amount: float
    '''Transaction amount.'''

    description: str
    '''Detailed description of the transaction.'''

    type: str
    '''Type of transaction (e.g., credit, debit).'''

    timestamp: str
    '''Date and time when the transaction occurred.'''



from depoc.resources.methods import Create, Retrieve, Update, Delete
from depoc.objects.member import MemberObject


class Member(
    Create[MemberObject],
    Retrieve[MemberObject],
    Update[MemberObject],
    Delete[MemberObject],
):
    obj = MemberObject
    endpoint = 'members'
    label = 'member'



from typing import Any
from requests import Response

from ._error import APIError


def _handle_response(response: Response) -> dict[str, Any]:
    status = response.status_code
    content_type = response.headers.get('Content-Type')
    not_json = content_type == 'text/html; charset=utf-8'

    if status == 404 and not_json:
        raise APIError('The requested resource was not found', 404)

    data: dict[str, Any] = response.json()

    if 'error' in data:
        error = data.get('error', {})
        message = error.get('message')
        status = error.get('status')
        details = error.get('details')
        raise APIError(message, status, details)
    
    return data



import json
import depoc
import click

from rich.console import Console
from rich.panel import Panel
from rich.text import Text
from rich.align import Align

from depoc.utils._error import APIError

console = Console()


@click.command()
@click.option('--username', prompt=True, required=True)
@click.password_option(required=True, confirmation_prompt=False)
def login(username: str, password: str) -> None :
    ''' Enter in your account '''
    auth = depoc.Connection(username, password)
    
    try:
        depoc.token = auth.token
        console.clear()
        title = Text(
            "🔐 LOGIN SUCCESSFUL",
            justify="center",
            style="bold green"
        )
        console.print(Align.center(Panel(title, expand=False)))

        with open(depoc.token_path, 'w') as f:
            json.dump({'token': auth.token}, f)
            
    except APIError as e:
        console.clear()
        title = Text(
            f"🚨 LOGIN FAILED",
            justify="center",
            style="bold red"
        )
        console.print(Align.center(Panel(title, expand=False)))
        console.print(Align.center(str(e.message)))



import depoc
import click
import sys
import time

from typing import Any

from rich.prompt import Prompt
from rich.panel import Panel
from rich.console import Console

from ..utils._response import _handle_response
from ..utils._format import spinner, page_summary, _format_transactions


client = depoc.DepocClient()
console = Console()


@click.group(invoke_without_command=True)
@click.pass_context
@click.option('-l', '--limit', default=50)
@click.option('-p', '--page', default=0)
@click.option('-b', '--bank')
def transaction(ctx, limit: int, page: int, bank: str) -> None:
    ''' Manage financial transactions '''
    if ctx.invoked_subcommand is None:
        service = client.financial_transactions.all

        if response := _handle_response(service, limit=limit, page=page):
            results = response.results

            if bank:
                results = [
                    obj for obj in results if obj.account.name == bank.title()
                ]
                # This page summary data needs to be
                # reviewed to accommodate different cases
                results_count = len(results)
                click.echo((
                    f'\n[Page {1}/{1}] '
                    f'Showing {results_count} results '
                    f'(Total: {results_count})\n'
                ))
            else:
                page_summary(response)

            for obj in results:
                title = f'\n{obj.account.name} {obj.type}'.upper()
                _format_transactions(obj, title)

@transaction.command
@click.option('-c', '--credit', is_flag=True)
@click.option('-d', '--debit', is_flag=True)
@click.option('-t', '--transfer', is_flag=True)
def create(
    credit: bool,
    debit: bool,
    transfer: bool,
) -> None:
    if not any([credit, debit, transfer]):
        message = (
            '🚨 Inform a type of transaction: '
            '-c (credit), -d (debit) or -t (transfer).'
        )
        console.print((
            '🚨 Inform a type of transaction:'
            '\n🏧 [bold][-t][/bold] [bold][--transfer][/bold]'
            '\n🏧 [bold][-c][/bold] [bold][--credit][/bold]'
            '\n🏧 [bold][-d][/bold] [bold][--debit][/bold]'
        ))
        sys.exit(0)

    panel = Panel('[bold]+ ADD NEW TRANSACTION')
    console.print(panel)

    data: dict[str, Any] = {}
    data.update({'amount': Prompt.ask('💰 Amount R$', default=None)})
    console.rule('',style=None, align='left')
    data.update({'account': Prompt.ask('🏦 Account', default=None)})
    console.rule('',style=None, align='left')
    data.update({'send_to': Prompt.ask('🏦 Send to')}) if transfer else None
    console.rule('',style=None, align='left') if transfer else None
    data.update({'description': Prompt.ask('🗒️  Description', default=None)})
    console.rule('',style=None, align='left')
    data.update({'category': Prompt.ask('📂 Category', default=None)})
    console.rule('',style=None, align='left')
    data.update({'contact': Prompt.ask('👤 Contact', default=None)})
    console.rule('',style=None, align='left')

    if credit:
        data.update({'type': 'credit'})
    elif debit:
        data.update({'type': 'debit'})
    elif transfer:
        data.update({'type': 'transfer'})

    service = client.financial_transactions.create

    if obj := _handle_response(service, data):
        title = f'\n{obj.account.name} {obj.type}'.upper()
        _format_transactions(obj, title)

@transaction.command
@click.argument('id')
def get(id: str) -> None:
    ''' Retrieve an specific transaction. '''
    service = client.financial_transactions.get
    if obj := _handle_response(service, resource_id=id):
        title = f'\n{obj.account.name} {obj.type}'.upper()
        _format_transactions(obj, title)

@transaction.command
@click.argument('ids', nargs=-1)
def delete(ids: str) -> None:
    ''' Delete an specific transaction. '''
    service = client.financial_transactions.delete

    while True:
        prompt = click.style('Proceed to deletion? [y/n] ', fg='red')
        confirmation = input(prompt)
        if confirmation == 'n':
            sys.exit(0)
        elif confirmation == 'y':
            break

    if len(ids) > 1:
        spinner()

    for id in ids:
        time.sleep(0.5)
        if _handle_response(service, resource_id=id):
            console.print('✅ Transaction deleted')

@transaction.command
@click.argument('search', required=False)
@click.option('-d', '--date')
@click.option('-s', '--start-date')
@click.option('-e', '--end-date')
@click.option('-b', '--bank')
@click.option('-l', '--limit', default=50)
@click.pass_context
def search(
    ctx,
    search: str,
    date: str,
    start_date: str,
    end_date: str,
    bank: str,
    limit: int,
    ) -> None:
    ''' Filter transactions. '''
    if not any([search, date, start_date, end_date]):
        click.echo(ctx.get_help())
        sys.exit(0)

    service = client.financial_transactions.filter

    if response := _handle_response(
        service,
        search=search,
        date=date,
        start_date=start_date,
        end_date=end_date,
        limit=limit,
    ):
        results = response.results
        if bank:
            results = [
                obj for obj in results if obj.account.name == bank.title()
            ]
            # This page summary data needs to be
            # reviewed to accommodate different cases
            results_count = len(results)
            click.echo((
                f'\n[Page {1}/{1}] '
                f'Showing {results_count} results '
                f'(Total: {results_count})\n'
            ))
        else:
            page_summary(response)

        for obj in results:
            title = f'\n{obj.account.name} {obj.type}'.upper()
            _format_transactions(obj, title)



from .base import DepocObject


class SupplierObject(DepocObject):
    '''
    Represents a supplier with identification, contact details,
    registration numbers, and status information.
    '''

    id: str
    ''' Unique identifier for the business. '''
    legal_name: str
    ''' Registered legal name of the business. '''
    trade_name: str
    ''' Commercial name used by the business. '''
    cnpj: str
    ''' Brazilian National Registry of Legal Entities (CNPJ). '''
    ie: str
    ''' State Registration Number (Inscrição Estadual). '''
    im: str
    ''' Municipal Registration Number (Inscrição Municipal). '''
    code: str
    ''' Unique internal supplier code. '''
    phone: str
    ''' Primary contact phone number. '''
    email: str
    ''' Contact email address. '''
    postcode: str
    ''' Postal code of the supplier's address. '''
    city: str
    ''' City where the supplier resides. '''
    state: str
    ''' State where the supplier resides. '''
    address: str
    ''' Full residential address of the supplier. '''
    notes: str
    ''' Additional notes or comments about the supplier. '''
    is_active: bool
    ''' Indicates if the supplier account is currently active. '''
    created_at: str
    ''' Timestamp of when the supplier record was created. '''
    updated_at: str
    ''' Timestamp of when the supplier record was last updated. '''



from depoc.resources.methods import Finder, Create, Retrieve, Update, Delete
from depoc.objects.finance import (
    FinancialAccountObject,
    FinancialCategoryObject,
    FinancialTransactionObject,
)


class FinancialAccount(
    Create[FinancialAccountObject],
    Retrieve[FinancialAccountObject],
    Update[FinancialAccountObject],
    Delete[FinancialAccountObject],
):
    obj = FinancialAccountObject
    endpoint = 'finance/accounts'
    label = 'account'


class FinancialCategory(
    Create[FinancialCategoryObject],
    Retrieve[FinancialCategoryObject],
    Update[FinancialCategoryObject],
    Delete[FinancialCategoryObject],
):
    obj = FinancialCategoryObject
    endpoint = 'finance/categories'
    label = 'category'


class FinancialTransaction(
    Finder[FinancialTransactionObject],
    Create[FinancialTransactionObject],
    Retrieve[FinancialTransactionObject],
    Update[FinancialTransactionObject],
    Delete[FinancialTransactionObject],
):
    obj = FinancialTransactionObject
    endpoint = 'finance/transactions'
    label = 'transaction'



import depoc
import click

from .utils._response import _handle_response
from .utils._format import _format_contact, page_summary


client = depoc.DepocClient()


@click.group(invoke_without_command=True)
@click.pass_context
@click.argument('search', required=False)
@click.option('-l', '--limit', default=50)
@click.option('-p', '--page', default=0)
def contact(cxt, limit: int, page: int, search: str) -> None:
    ''' Contacts - retrieve all and filter. '''
    if cxt.invoked_subcommand is None:
        service = client.contacts.all
        response = _handle_response(service, limit=limit, page=page)

        if search:
            service = client.contacts.filter
            response = _handle_response(service, search=search, limit=limit)

        if response:
            page_summary(response)

            for obj in response.results:
                if hasattr(obj, 'customer'):
                    title = f'{obj.customer.name}'
                    obj = obj.customer
                elif hasattr(obj, 'supplier'):
                    title = f'{obj.supplier.legal_name}'
                    obj = obj.supplier
                
                _format_contact(obj, title)



from .base import DepocObject


class UserObject(DepocObject):
    ''' Represents a user resource '''

    id: str
    ''' Unique identifier of the user. '''
    name: str
    ''' Full name of the user. '''
    email: str
    ''' User's email address. '''
    username: str
    ''' Unique username of the user. '''
    is_active: bool
    ''' Indicates whether the user account is active. '''
    is_staff: bool
    ''' Indicates whether the user has staff privileges. '''
    is_superuser: bool
    ''' Indicates whether the user has superuser privileges. '''
    last_login: str
    ''' Timestamp of the user's last login. '''
    date_joined: str
    ''' Timestamp of when the user registered in Depoc. '''



import depoc
import click

from rich.console import Console
from rich.panel import Panel
from rich.table import Table

from ..utils._response import _handle_response


client = depoc.DepocClient()
console = Console()

def _format_bank(obj, update: bool = False):
    table = Table(
        show_header=False,
        show_footer=True,
        box=None,
        expand=True,
        caption=f'{obj.id}',
        caption_justify='left'
        )
    
    table.add_column('', justify='left', no_wrap=True)
    table.add_column('', justify='right', no_wrap=True)
    balance = float(obj.balance)
    table.add_row(f'', f'[bold]R${balance:,.2f}')

    if update:
        border_style = 'green'
    elif not obj.is_active:
        border_style = 'red'
    else:
        border_style = 'none'

    panel = Panel(
        table,
        title=f'[bold]💰 {obj.name.upper()}',
        title_align='left',
        border_style=border_style,
    )

    console.print(panel)


@click.group(invoke_without_command=True)
@click.pass_context
def bank(ctx) -> None:
    ''' Manage bank accounts '''
    if ctx.invoked_subcommand is None:
        service = client.financial_accounts.all
        total_balance: float = 0
        if response := _handle_response(service):
            results = sorted(
                 response.results,
                 key=lambda result : result.balance,
                 reverse=True,
            )

            for obj in results:
                total_balance += float(obj.balance)
                _format_bank(obj)

            format_total_balance = f'R${total_balance:,.2f}'
            message = f'\n{'💵 Total Balance: ' + format_total_balance}\n'
            click.echo(message)

@bank.command
@click.argument('name')
def create(name: str) -> None:
    ''' Create a new bank account. '''
    data = {'name': name}
    service = client.financial_accounts.create

    if obj := _handle_response(service, data):
        _format_bank(obj)

@bank.command
@click.argument('id')
def get(id: str) -> None:
    ''' Retrieve an specific bank account. '''
    service = client.financial_accounts.get

    if obj := _handle_response(service, resource_id=id):
        _format_bank(obj)

@bank.command
@click.argument('id')
@click.argument('name')
@click.option('-A', '--activate', is_flag=True)
def update(id: str, name: str, activate: bool = False) -> None:
    ''' Update a bank account. '''
    data = {'name': name}

    # I want to reactivate a bank without having to provide a name
    if activate:
        data.update({'is_active': True})

    service = client.financial_accounts.update

    if obj := _handle_response(service, data, id):
        _format_bank(obj, update=True)

@bank.command
@click.argument('id')
def delete(id: str) -> None:
    ''' Delete a bank account. '''
    service = client.financial_accounts.delete

    if _handle_response(service, resource_id=id):
        console.print('✅ Bank account inactivated')



from depoc.resources.methods import Create, Retrieve, Update, Delete
from depoc.objects.customer import CustomerObject


class Customer(
    Create[CustomerObject],
    Retrieve[CustomerObject],
    Update[CustomerObject],
    Delete[CustomerObject],
):
    obj = CustomerObject
    endpoint = 'contacts/customers'
    label = 'customer'



from depoc.resources.methods import Finder, Create, Retrieve, Update, Delete
from depoc.objects.product import (
    ProductObject,
    ProductCategoryObject,
    ProductCostObject,
)


class Product(
    Finder[ProductObject],
    Create[ProductObject],
    Retrieve[ProductObject],
    Update[ProductObject],
    Delete[ProductObject],
):
    obj = ProductObject
    endpoint = 'products'
    label = 'product'


class ProductCategory(
    Finder[ProductCategoryObject],
    Create[ProductCategoryObject],
    Retrieve[ProductCategoryObject],
    Update[ProductCategoryObject],
    Delete[ProductCategoryObject],
):
    obj = ProductCategoryObject
    endpoint = 'products/categories'
    label = 'category'


class ProductCost(
    Finder[ProductCostObject],
    Create[ProductCostObject],
    Retrieve[ProductCostObject],
    Update[ProductCostObject],
    Delete[ProductCostObject],
):
    obj = ProductCostObject
    endpoint = 'products/<id>/costs'
    label = 'cost'



from .base import DepocObject


class CustomerObject(DepocObject):
    '''
    Represents a customer entity with personal details,
    contact information, and transaction history.
    '''

    id: str
    ''' Unique identifier for the customer '''
    name: str
    ''' Full name of the customer. '''
    alias: str
    ''' Preferred name or nickname of the customer. '''
    cpf: str
    ''' Brazilian individual taxpayer registry identification (CPF). '''
    gender: str
    ''' Gender of the customer. '''
    amount_spent: float
    ''' Total amount spent by the customer. '''
    number_of_orders: int
    ''' Total number of orders placed by the customer. '''
    code: str
    ''' Unique internal customer code. '''
    phone: str
    ''' Primary contact phone number. '''
    email: str
    ''' Contact email address. '''
    postcode: str
    ''' Postal code of the customer's address. '''
    city: str
    ''' City where the customer resides. '''
    state: str
    ''' State where the customer resides. '''
    address: str
    ''' Full residential address of the customer. '''
    notes: str
    ''' Additional notes or comments about the customer. '''
    is_active: bool
    ''' Indicates if the customer account is currently active. '''
    created_at: str
    ''' Timestamp of when the customer record was created. '''
    updated_at: str
    ''' Timestamp of when the customer record was last updated. '''



import depoc
import click
import sys
import time

from typing import Any

from rich.console import Console
from rich.console import group
from rich.panel import Panel
from rich.table import Table

from ..utils._response import _handle_response
from ..utils._format import spinner, _format_category


client = depoc.DepocClient()
console = Console()


@group()
def get_tables(tables: list):
    for table in tables:
        yield table


@click.group(invoke_without_command=True)
@click.pass_context
@click.option('-l', '--limit', default=50)
@click.option('-p', '--page', default=0)
def category(ctx, limit: int, page: int) -> None:
    ''' Manage financial categories '''
    if ctx.invoked_subcommand is None:
        service = client.financial_categories.all

        if response := _handle_response(service, limit=limit, page=page):
            # I want to include the is_active
            # attr into the categories dict
            categories: dict[str, dict] = {}
            for obj in response.results:
                if not obj.parent:
                    categories.update({obj.name: {'id': obj.id}})

            for obj in response.results:
                if obj.parent:
                    parent = obj.parent
                    while parent:
                        if parent.name not in categories.keys():
                            child = categories[parent.parent.name][parent.name]
                            if child:
                                child.update({
                                    'id': parent.id,
                                    obj.name: {'id': obj.id}
                                })
                        elif obj.parent.name in categories.keys():
                            categories[parent.name].update({
                                obj.name: {'id': obj.id}
                            })
                        parent = parent.parent

            for k, v in categories.items():
                table = Table(
                    show_header=True,
                    show_footer=True,
                    box=None,
                    expand=True,
                    )
                
                table.add_column('', justify='left', no_wrap=True)
                table.add_column('', justify='right', no_wrap=True)

                tables = [table]

                for name, value in v.items():
                    if isinstance(value, dict):
                        if len(value) > 1:
                            sub = Table(
                                show_header=True,
                                show_footer=True,
                                box=None,
                                expand=True,
                            )

                            sub.add_column(name, justify='left', no_wrap=True)
                            sub.add_column(value['id'], justify='right', no_wrap=True)

                            for a, b in value.items():
                                if a != 'id':
                                    sub.add_row(
                                        f'[bright_black]{a}',
                                        f'[bright_black]{b['id']}'
                                    )
                            tables.append(sub)
                        else:
                            table.add_row(name, value['id'])

                group = get_tables(tables)

                profile = Panel(
                    group,
                    title=f'[bold]{k.upper()}',
                    title_align='left',
                    subtitle=v['id'],
                    subtitle_align='left',
                )

                console.print(profile)

@category.command
@click.argument('name')
@click.option('-p', '--parent', help='Inform the Parent Caregory if any.')
def create(name: str, parent: str) -> None:
    ''' Create a new category. '''
    data: dict[str, Any] = {'name': name}
    data.update({'parent': parent}) if parent else None

    service = client.financial_categories.create
    if obj := _handle_response(service, data):
        _format_category(obj, highlight=True)


@category.command
@click.argument('id')
def get(id: str) -> None:
    ''' Retrieve an specific category '''
    service = client.financial_categories.get
    if obj := _handle_response(service, resource_id=id):
        _format_category(obj)

@category.command
@click.argument('id')
@click.option('--name', help='Inform the new name for the Category.')
@click.option('--parent', help='Inform the Parent Caregory if any.')
@click.option('--activate', is_flag=True, help='Activate category.')
def update(id: str, name: str, parent: str, activate: bool) -> None:
    ''' Update a category '''
    data: dict[str, Any] = {}
    data.update({'name': name}) if name else None
    data.update({'parent': parent}) if parent else None
    data.update({'is_active': True}) if activate else None

    service = client.financial_categories.update

    if obj := _handle_response(service, data, id):
        _format_category(obj, highlight=True)

@category.command
@click.argument('ids', nargs=-1)
def delete(ids: str) -> None:
    ''' Delete a category '''
    service = client.financial_categories.delete

    while True:
        prompt = click.style('Proceed to deletion? [y/n] ', fg='red')
        confirmation = input(prompt)
        if confirmation == 'n':
            sys.exit(0)
        elif confirmation == 'y':
            break

    if len(ids) > 1:
        spinner()

    for id in ids:
        time.sleep(0.5)
        if _handle_response(service, resource_id=id):
            console.print('✅ Category inactivated')



from .base import DepocObject


class MemberObject(DepocObject):
    ''' Represents a member of the business '''

    id: str
    ''' Unique identifier of the member. '''
    name: str
    ''' Full name of the member. '''
    email: str
    ''' Member's email address. '''
    phone: str
    ''' Member's phone number. '''
    cpf: str
    ''' Brazilian individual taxpayer registry identification (CPF). '''
    date_of_birth: str
    ''' Member's date of birth in YYYY-MM-DD format. '''
    role: str
    ''' Position or job title of the member within the business. '''
    hire_date: str
    ''' Date the member was hired, in YYYY-MM-DD format. '''
    salary: float
    ''' Member's salary. '''
    has_access: bool
    ''' Indicates if the member has system access. '''
    is_active: bool
    ''' Indicates if the member is currently active in the organization. '''
    credential: dict
    ''' Access credentials. '''



from depoc.resources.methods import Create, Retrieve, Update, Delete
from depoc.objects.business import BusinessObject


class Business(
    Create[BusinessObject],
    Retrieve[BusinessObject],
    Update[BusinessObject],
    Delete[BusinessObject]
):
    obj = BusinessObject
    endpoint = 'business'
    label = 'business'



import depoc
import click
import sys

from rich.console import Console
from rich.panel import Panel
from rich.table import Table

from .utils._response import _handle_response


client = depoc.DepocClient()


@click.command
@click.option('-d', '--date')
@click.option('-s', '--start-date')
@click.option('-e', '--end-date')
@click.pass_context
def balance(ctx, date: str, start_date: str, end_date: str) -> None:
    ''' Balance for a specific period. '''
    if not any([date, start_date, end_date]):
        date = 'week'
        params = {'date': date}
    elif date and not any([start_date, end_date]):
        params = {'date': date}
    elif start_date and end_date:
        params = {'start_date': start_date, 'end_date': end_date}
    else:
        console.print(
            '📅 [bold][-s] [--start-date] \n[/bold]'
            '📅 [bold][-e] [--end-date] \n[/bold]'
            '🚨 Must be specified as a pair'
        )
        sys.exit()
    
    caption = date
    if start_date and end_date:
        caption = f'{start_date} → {end_date}'
    elif date in ('week', 'month'):
        caption = f"this {date}'s balance"
    elif date == 'today':
        caption = f"today's balance"

    income: float = 0
    expenses: float = 0
    balance: float = 0
    
    service = client.financial_transactions.filter

    if response := _handle_response(
        service,
        **params,
        ):
        for obj in response.results:
            if obj.type == 'credit':
                income += float(obj.amount)
            elif obj.type == 'debit':
                expenses += float(obj.amount)

        balance = round(income + expenses, 2)

        table = Table(
            show_header=True,
            show_footer=True,
            box=None,
            expand=True,
            caption=caption,
            title=f'[bold]R${balance:,.2f}'
            )
        
        table.add_column('', justify='left', no_wrap=True)
        table.add_column('', justify='right', no_wrap=True)
        table.add_row('📈 Income', f'R${income:,.2f}')
        table.add_row('📉 Expenses', f'R${abs(expenses):,.2f}')

        border_style = 'red' if balance < 0 else 'green'

        profile = Panel(
            table,
            title=f'[bold]BALANCE',
            title_align='left',
            border_style=border_style,
        )

        console = Console()
        console.print(profile)


import depoc
import click
import sys

from .utils._response import _handle_response
from .utils._format import _format_profile


client = depoc.DepocClient()


@click.group
def account() -> None:
    ''' Manage user account '''
    pass

@account.command
@click.option('-n', '--name')
@click.option('-e', '--email')
@click.option('-u', '--username')
def update(name: str, email: str, username: str) -> None:
    data = {}
    data.update({'name': name}) if name else None
    data.update({'email': email}) if email else None
    data.update({'username': username}) if username else None

    service = client.accounts.update

    if obj := _handle_response(service, data):
        _format_profile(obj, '[green]UPDATED', update=True)

@account.command
def delete() -> None:
    service = client.accounts.delete

    while True:
        prompt = click.style('Proceed to deletion? [y/n] ', fg='red')
        confirmation = input(prompt)
        if confirmation == 'n':
            sys.exit(0)
        elif confirmation == 'y':
            break

    if obj := _handle_response(service):
        _format_profile(obj, '[red]DEACTIVATED', delete=True)
